﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Windows;
using System.Windows.Resources;

namespace XamGridExpand
{
    public class ChildNode
    {
        public ChildNode()
        {
            ChildrenCount = 0;
            Children = new ObservableCollection<ChildNode>();
            AnotherDetails = new ObservableCollection<AnotherRowDetail>();
            IsChainName = false;
        }
        public string ID
        {
            get;
            set;
        }
        public string Name
        {
            get;
            set;
        }
        public string ParentID
        {
            get;
            set;
        }
        public int ChildrenCount
        {
            get;
            set;
        }
        public bool IsChainName
        {
            get;
            set;
        }
        public ObservableCollection<ChildNode> Children
        {
            get;
            set;
        }
        public ObservableCollection<AnotherRowDetail> AnotherDetails
        {
            get;
            set;
        }

        private IEnumerable<string> GetItemsFromCSVFile(string filename)
        {
            StreamResourceInfo csvsri = Application.GetResourceStream(new Uri(filename, UriKind.RelativeOrAbsolute));
            using (StreamReader streamReader = new StreamReader(csvsri.Stream))
            {
                string currentline = streamReader.ReadLine();

                while (currentline != null)
                {
                    currentline = currentline.Trim();
                    if (currentline.Length > 0)
                        yield return currentline;
                    currentline = streamReader.ReadLine();
                }
            }
        }

        #region For preparing list from file data
        public List<ChildNode> GetData(string sFileName)
        {
            List<ChildNode> resultnode = new List<ChildNode>();
            try
            {
                ChildNode parentnode = null;

                string[] currentlinetokens = null;

                string id = String.Empty;
                string name = String.Empty;
                string parentid = String.Empty;
                int iChildrenCnt = 0;

                /* We are assuming that the data in the csv file is in order with parentdata first. */
                foreach (string item in GetItemsFromCSVFile(sFileName))
                {
                    currentlinetokens = item.Split(',');

                    id = currentlinetokens[0];
                    name = currentlinetokens[1];
                    parentid = (currentlinetokens[2].ToLower() != "null") ? currentlinetokens[2] : "";
                    parentid = (currentlinetokens[2].ToLower() != "null") ? currentlinetokens[2] : "";
                    iChildrenCnt = (currentlinetokens[3].ToLower() != "null") ? Convert.ToInt32(currentlinetokens[3]) : 0;

                    parentnode = new ChildNode() { ID = id, Name = name, ParentID = parentid, ChildrenCount = iChildrenCnt };

                    resultnode.Add(parentnode);
                }
            }
            catch (Exception)
            {
            }
            return resultnode;
        }
        #endregion

        public ObservableCollection<ChildNode> GetTheDataByParentID(string strParentID, List<ChildNode> lstAllData)
        {
            ObservableCollection<ChildNode> lstResult = new ObservableCollection<ChildNode>();
            ChildNode objNode = new ChildNode();
            var Result = (from p in lstAllData
                          where p.ParentID == strParentID
                          select p).ToList();

            for (int i = 0; i < Result.ToList().Count; i++)
            {
                if (Result[i].ParentID == strParentID)
                {
                    objNode = new ChildNode();
                    if (Result[i].ChildrenCount > 0)
                    {
                        objNode.Children.Add(new ChildNode());
                    }
                    objNode.ID = Result[i].ID;
                    objNode.Name = Result[i].Name;
                    objNode.ParentID = strParentID;
                    objNode.ChildrenCount = Result[i].ChildrenCount;
                    lstResult.Add(objNode);
                }
            }
            return lstResult;
        }
    }

    public class AnotherRowDetail
    {
        public AnotherRowDetail()
        {
            DateOfCreation = DateTime.Today;
            Details = new ObservableCollection<AnotherRowDetail>();
            IsChainName = false;
        }
        public string ID
        {
            get;
            set;
        }
        public string Name
        {
            get;
            set;
        }
        public string AnotherParentID
        {
            get;
            set;
        }
        public DateTime DateOfCreation
        {
            get;
            set;
        }
        public bool IsChainName
        {
            get;
            set;
        }
        public ObservableCollection<AnotherRowDetail> Details
        {
            get;
            set;
        }
    }
}

